
"""
Runs whisper api.
"""

import shutil
import sys
import time
from pathlib import Path
import subprocess
from typing import Optional
from filelock import FileLock

from isolated_environment import IsolatedEnvironment  # type: ignore

HERE = Path(__file__).parent
ENV: Optional[IsolatedEnvironment] = None
CUDA_AVAILABLE: Optional[bool] = None
ENV_LOCK = FileLock(HERE / "whisper_env.lock")

# Set the versions
TENSOR_VERSION = "2.1.2"
CUDA_VERSION = "cu121"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"


# for insanely fast whisper, use:
#   pipx install insanely-fast-whisper --python python3.11

def has_nvidia_smi() -> bool:
    """Returns True if nvidia-smi is installed."""
    return shutil.which("nvidia-smi") is not None


def get_environment() -> IsolatedEnvironment:
    """Returns the environment."""
    global ENV  # pylint: disable=global-statement
    if ENV is not None:
        return ENV
    venv_dir = HERE / "venv" / "whisper"
    env = IsolatedEnvironment(venv_dir)
    if not venv_dir.exists():
        env.install_environment()
        if has_nvidia_smi():
            env.pip_install(f"torch=={TENSOR_VERSION}", extra_index=EXTRA_INDEX_URL)
        else:
            env.pip_install(f"torch=={TENSOR_VERSION}")
        env.pip_install("openai-whisper")
    ENV = env
    return env



def get_computing_device() -> str:
    """Get the computing device."""
    with ENV_LOCK:
        global CUDA_AVAILABLE  # pylint: disable=global-statement
        if CUDA_AVAILABLE is None:
            iso_env = get_environment()
            env = iso_env.environment()
            py_file = HERE / "cuda_available.py"
            rtn = subprocess.run([
                "python", py_file
            ], check=False, env=env).returncode
            CUDA_AVAILABLE = rtn == 0
        return "cuda" if CUDA_AVAILABLE else "cpu"

def run_whisper(  # pylint: disable=too-many-arguments
    input_wav: Path,
    device: str,
    model: str,
    output_dir: Path,
    task: str,
    language: str,
    other_args: Optional[list[str]]
) -> None:
    """Runs whisper."""
    iso_env = get_environment()
    cmd_list = []
    if sys.platform == "win32":
        # Set the text mode to UTF-8 on Windows.
        cmd_list.extend(["chcp", "65001", "&&"])
    cmd_list.append("whisper")
    cmd_list.append(f'"{input_wav}"')
    cmd_list.append("--device")
    cmd_list.append(device)
    cmd_list.append("--model")
    cmd_list.append(model)
    cmd_list.append(f'--output_dir "{output_dir}"')
    cmd_list.append("--task")
    cmd_list.append(task)
    if language:
        cmd_list.append(f'--language "{language}"')

    if other_args:
        cmd_list.extend(other_args)
    # Remove the empty strings.
    cmd_list = [x.strip() for x in cmd_list if x.strip()]
    cmd = " ".join(cmd_list)
    sys.stderr.write(f"Running:\n  {cmd}\n")
    proc = subprocess.Popen(  # pylint: disable=consider-using-with
        cmd, shell=True, universal_newlines=True,
        env=iso_env.environment(),
        encoding="utf-8"
    )
    while True:
        rtn = proc.poll()
        if rtn is None:
            time.sleep(0.25)
            continue
        if rtn != 0:
            msg = f"Failed to execute {cmd}\n "
            raise OSError(msg)
        break



"""
    Entry point for running the transcribe-anything prgram.
"""

# flake8: noqa E501

import argparse
import sys
import os
import traceback

from transcribe_anything.api import transcribe
from transcribe_anything.whisper import get_computing_device
from transcribe_anything.parse_whisper_options import parse_whisper_options

os.environ["PYTHONIOENCODING"] = "utf-8"

WHISPER_MODEL_OPTIONS = [
    "tiny",
    "tiny.en",
    "base",
    "base.en",
    "small",
    "small.en",
    "medium",
    "medium.en",
    "large-legacy",
    "large",
    "large-v2",
    "large-v3"
]

def main() -> int:
    """Main entry point for the command line tool."""
    whisper_options = parse_whisper_options()
    device = get_computing_device()
    help_str = (
        f'transcribe_anything is using a "{device}" device.'
        " Any unrecognized args are assumed to be for whisper"
        " ai and will be passed as is to whisper ai."
    )
    parser = argparse.ArgumentParser(description=help_str)
    parser.add_argument(
        "url_or_file",
        help="Provide file path or url (includes youtube/facebook/twitter/etc)",
    )
    parser.add_argument(
        "--output_dir",
        help="Provide output directory name,d efaults to the filename of the file.",
        default=None,
    )
    parser.add_argument(
        "--model",
        help="name of the Whisper model to us",
        default="small",
        choices=WHISPER_MODEL_OPTIONS,
    )
    parser.add_argument(
        "--task",
        help="whether to perform transcription or translation",
        default="transcribe",
        choices=whisper_options["task"],
    )
    parser.add_argument(
        "--language",
        help="language to the target audio is in, default None will auto-detect",
        default=None,
        choices=[None] + whisper_options["language"],
    )
    parser.add_argument(
        "--device",
        help="device to use for processing, None will auto select CUDA if available or else CPU",
        default=None,
        choices=[None, "cpu", "cuda", "insane"],
    )
    parser.add_argument(
        "--embed",
        help="whether to embed the translation file into the output file",
        action="store_true",
    )
    # add extra options that are passed into the transcribe function
    args, unknown = parser.parse_known_args()
    if args.model == "large-legacy":
        args.model = "large"
    elif args.model == "large":
        args.model = "large-v3"

    if unknown:
        print(f"Unknown args: {unknown}")
    print(f"Running transcribe_audio on {args.url_or_file}")
    try:
        transcribe(
            url_or_file=args.url_or_file,
            output_dir=args.output_dir,
            model=args.model if args.model != "None" else None,
            task=args.task,
            language=args.language if args.language != "None" else None,
            device=args.device,
            embed=args.embed,
            other_args=unknown,
        )
    except KeyboardInterrupt:
        print("KeyboardInterrupt")
        return 1
    except Exception as e:  # pylint: disable=broad-except
        stack = traceback.format_exc()
        sys.stderr.write(f"Error: {e}\n{stack}\nwhile processing {args.url_or_file}\n")
        return 1
    return 0


if __name__ == "__main__":
    # push sys argv prior to call
    sys.argv.append("test.wav")
    sys.argv.append("--model")
    sys.argv.append("large")
    sys.argv.append('--initial_prompt "What is your name?"')
    sys.exit(main())
